#!/usr/bin/perl
use warnings;
use strict;
use Carp;
use Pod::Usage;
use Getopt::Long;

our $VERSION = 			'$Revision: 341 $';
our $LAST_CHANGED_DATE =	'$LastChangedDate: 2010-05-01 19:33:32 -0400 (Sat, 01 May 2010) $';

our ($verbose, $help, $man);
our ($reportfile, $prefix, $suffix, $numeric_name, $comma, $tolerate);

GetOptions ('verbose'=>\$verbose, 'help'=>\$help, 'man'=>\$man, 'prefix=s'=>\$prefix, 'suffix=s'=>\$suffix, 'numeric_name'=>\$numeric_name, 'comma'=>\$comma, 'tolerate'=>\$tolerate) or pod2usage ();

$help and pod2usage (-verbose=>1, -exitval=>1, -output=>\*STDOUT);
$man and pod2usage (-verbose=>2, -exitval=>1, -output=>\*STDOUT);
@ARGV or pod2usage (-verbose=>0, -exitval=>1, -output=>\*STDOUT);
@ARGV == 1 or pod2usage ("Syntax error");

($reportfile) = @ARGV;
$prefix ||= '';
$suffix ||= '';

splitIlluminaReport ($reportfile, $prefix, $suffix, $numeric_name, $comma);



sub splitIlluminaReport {
	my ($reportfile, $prefix, $suffix, $numeric_name, $comma) = @_;
	my ($count_line, $count_file, $pre, $name_index, $sample_index, $lrr_index, $baf_index) = (0, 0, 'NA');
	my (@field);
	
	open (REPORT, $reportfile) or confess "Error: cannot read from input report file $reportfile: $!\n";
	while (<REPORT>) {
		$count_line++;
		m/^\[Data\]/ and last;
		$count_line > 1000 and confess "Error: after reading 1000 lines in $reportfile, still cannot find [Data] section. The $reportfile file may not be in Illumina report format.\n";
	}
	
	$_ = <REPORT>;
	s/[\r\n]+$//;
	$count_line++;
	@field = $comma ? (split (/,/, $_)) : (split (/\t/, $_));
	@field >= 4 or confess "Error: invalid header line (at least 4 tab-delimited fields, including 'SNP Name', 'Sample ID', 'B Allele Freq', 'Log R Ratio' expected) in report file $reportfile: <$_>\n";
	
	for my $i (0 .. @field-1) {
		$field[$i] eq 'SNP Name' and $name_index = $i;
		$field[$i] eq 'Sample ID' and $sample_index = $i;
		$field[$i] eq 'B Allele Freq' and $baf_index = $i;
		$field[$i] eq 'Log R Ratio' and $lrr_index = $i;
	}
	defined $name_index or confess "Error: the 'SNP Name' field is not found in header line in report file $reportfile: <$_>\n";
	defined $sample_index or confess "Error: the 'Sample ID' field is not found in header line in report file $reportfile: <$_>\n";
	defined $baf_index or confess "Error: the 'B Allele Freq' field is not found in header line in report file $reportfile: <$_>\n";
	defined $lrr_index or confess "Error: the 'Log R Ratio' field is not found in header line in report file $reportfile: <$_>\n";
	
	while (<REPORT>) {
		s/[\r\n]+$//;
		$count_line++;
		@field = $comma ? (split (/,/, $_)) : (split (/\t/, $_));
		@field >= 4 or confess "Error: invalid data line (at least 4 tab-delimited fields expected) in report file $reportfile: <$_>\n";
		defined $field[$name_index] or confess "Error: the 'SNP Name' field is not found in data line in report file $reportfile: <$_>\n";
		defined $field[$sample_index] or confess "Error: the 'Sample ID' field is not found in data line in report file $reportfile: <$_>\n";
		if (not defined $field[$baf_index]) {
			if ($tolerate) {
				print STDERR "WARNING: Skipping marker $field[$name_index] for $field[$sample_index] due to lack of BAF information\n";
				next;
			} else {
				confess "Error: the 'B Allele Freq' field is not found in data line in report file $reportfile: <$_>\n";
			}
		}
		if (not defined $field[$lrr_index]) {
			if ($tolerate) {
				print STDERR "WARNING: Skipping marker $field[$name_index] for $field[$sample_index] due to lack of LRR information\n";
				next;
			} else {
				confess "Error: the 'Log R Ratio' field is not found in data line in report file $reportfile: <$_>\n";
			}
		}
		
		if ($field[$sample_index] eq $pre) {
			print OUT join ("\t", @field[$name_index, $lrr_index, $baf_index]), "\n";
		} else {
			$count_file++;
			my $outname;
			if ($numeric_name) {
				$outname = "$prefix"."split$count_file$suffix";
			} else {
				$outname = "$prefix$field[$sample_index]$suffix";
			}
			print STDERR "NOTICE: Writting to output signal file $outname\n";
			open (OUT, ">$outname") or confess "Error: cannot write to output file $outname:$!\n";
			print OUT "Name\t$field[$sample_index].Log R Ratio\t$field[$sample_index].B Allele Freq\n";
			print OUT join("\t",@field[$name_index, $lrr_index, $baf_index]), "\n";
		}
		$pre = $field[$sample_index];
	}
	print STDERR "NOITCE: Finished processing $count_line lines in report file $reportfile, and generated $count_file output signal intensity files\n";
}


=head1 SYNOPSIS

 split_illumina_report.pl [arguments] <reportfile>

 Optional arguments:
 	-v, --verbose			use verbose output
 	-h, --help			print help message
 	-m, --man			print complete documentation
 	    --prefix <string>		prefix of output file name
 	    --suffix <string>		suffix of output file name
 	    --numeric_name		use numeric file name (default: Sample ID is file name)
 	    --comma			fields are comma-delimited (default: fields are tab-delimited)
 	    --tolerate			tolerate records without LRR/BAF information
 	    

 Function: split the Illumina report file to individual signal intensity files

 Example: split_illumina_report.pl -prefix signal/ -suffix .txt HapMap.report
          split_illumina_report.pl -num HapMap.report

 Version: $LastChangedDate: 2010-05-01 19:33:32 -0400 (Sat, 01 May 2010) $

=head1 OPTIONS

=over 8

=item B<--help>

print a brief help message and exit

=item B<--man>

print the complete manual of how to use the program

=item B<--verbose>

use verbose output

=item B<--prefix>

specify the prefix for output file name. By default the file name is the "Sample 
ID" field in the report file.

=item B<--suffix>

specify the suffix for output file name. By default the file name is the "Sample 
ID" field in the report file.

=item B<--numeric_name>

specify that the output file name should be arranged in a numeric manner (such 
as split1, split2, split3, etc.).

=item B<--comma>

specify that the data fields in report file is comma-delimited. By default, the 
tab-delimited format is assumed.

=back

=head1 DESCRIPTION

This program is used to convert the "Report file" from Illumina BeadStudio 
software to a format that can be used by detect_cnv.pl program for CNV 
detection.

The report file should contain at least four tab-delimited fields, including 
'SNP Name', 'Sample ID', 'B Allele Freq', 'Log R Ratio'. If not, try to generate 
the report file again in BeadStudio, and make sure that these fields are 
selected to be exported.

For questions, comments or bug reports, please contact me at 
kai@openbioinformatics.org.

=cut